// Copyleft (cl) 1999, 2000 by Malekith / The Final Crash of Corp. System
// <malekith@topnet.pl> / <http://malekith.topnet.pl>
// This is part of gallery image viewer. See file copying-left for details.
// Created on Feb 26 2000
// $Id$

#include "main.h"

#ifdef aalib

#include "browser.h"
#include <aalib.h>

void Browser::init_aa()
{
	ctx = aa_autoinit(&aa_defparams);
	if (!ctx) {
		fprintf(stderr, "couldn't initialize aalib.\n");
		exit(1);
	}
	aa_autoinitkbd(ctx,0);
	screen_w = aa_imgwidth(ctx);
	screen_h = aa_imgwidth(ctx) * aa_mmheight(ctx) / aa_mmwidth(ctx);
	bitspp = 24; // realy grayscale
	bytespp = 3;
//	BYTESPERPIXEL = 3;
	scroll_step = 10;
}

aa_renderparams rendp;
void Browser::handle_aa_key(int key)
{
#define GAMMA_JUMP 0.05
#define BRIGHT_JUMP 1
#define CONTRAST_JUMP 1
	bool stats = false;
	switch (key) {
	case 'a':
		rendp = aa_defrenderparams;
		break;
	case 's':
		if (rendp.gamma >= GAMMA_JUMP)
			rendp.gamma -= GAMMA_JUMP;
		break;
	case 'x':
		rendp.gamma += GAMMA_JUMP;
		break;
	case 'z':
		switch (rendp.dither) {
		case AA_NONE: rendp.dither = AA_ERRORDISTRIB; break;
		case AA_ERRORDISTRIB: rendp.dither = AA_FLOYD_S; break;
		default:
		case AA_FLOYD_S: rendp.dither = AA_NONE; break;
		}
		break;
	case 'c':
		if (rendp.bright >= BRIGHT_JUMP)
			rendp.bright -= BRIGHT_JUMP;
		break;
	case 'd':
		if (rendp.bright <= 255 - BRIGHT_JUMP)
			rendp.bright += BRIGHT_JUMP;
		break;
	case 'f':
		if (rendp.contrast >= CONTRAST_JUMP)
			rendp.contrast -= CONTRAST_JUMP;
		break;
	case 'v':
		if (rendp.contrast <= 127 - CONTRAST_JUMP)
			rendp.contrast += CONTRAST_JUMP;
		break;
	case '\t':
		stats = true;
		break;
	default:
		return;
	}
	update_screen(false);
	if (stats) {
		aa_printf(ctx, 0, 0, AA_SPECIAL, _("gamma: %.2f, bright: %d, contrast: %d, dither: %s"),
		rendp.gamma, rendp.bright, rendp.contrast, 
		rendp.dither == AA_NONE ? "none" :
		rendp.dither == AA_ERRORDISTRIB ? "error-d" : "f-s");
		aa_flush(ctx);
	}
}

void Browser::kill_aa()
{
	aa_close(ctx);
}

void preinit_aa(int *argc, char **argv)
{
	rendp = aa_defrenderparams;
	aa_parseoptions(0, 0, argc, argv);
}

void Browser::read_keys_aa(int &key, int &shift)
{
	shift = false;
	key = aa_getkey(ctx, 0);
	if (!key)
		return;
	if (key >= 'A' && key <= 'Z')
		key ^= 'a' - 'A';
	switch (key) {
	case AA_ESC:
		key = 'q';
		break;
	case '[':
	case AA_LEFT:
		key = xleft;
		break;
	case ']':
	case AA_RIGHT:
		key = xright;
		break;
	case '\'':
	case AA_DOWN:
		key = xdown;
		break;
	case ';':
	case AA_UP:
		key = xup;
		break;
	case '\r':
		key = '\n';
		break;
	case '=':
		key = '+';
		break;
	case '!':
		key = '1';
		shift = true;
		break;
	case '@':
		key = '2';
		shift = true;
		break;
	case '#':
		key = '3';
		shift = true;
		break;
	case '$':
		key = '4';
		shift = true;
		break;
	case '%':
		key = '5';
		shift = true;
		break;
	case '^':
		key = '6';
		shift = true;
		break;
	case '&':
		key = '7';
		shift = true;
		break;
		//case '*': key = '8'; shift = true; break;
	case '(':
		key = '9';
		shift = true;
		break;
	case ')':
		key = '0';
		shift = true;
		break;
	}
}

void Browser::draw_progress_aa(int perc)
{
	aa_printf(ctx, 0, 0, AA_SPECIAL,
		  "|%c%c%c%c%c%c%c%c%c|%3d%%",
		  perc > 10 ? '#' : ':', perc > 20 ? '#' : ':',
		  perc > 30 ? '#' : ':', perc > 40 ? '#' : ':',
		  perc > 50 ? '#' : ':', perc > 60 ? '#' : ':',
		  perc > 70 ? '#' : ':', perc > 80 ? '#' : ':',
		  perc > 90 ? '#' : ':', perc);
	aa_flush(ctx);
}

bool Browser::get_halt_code_aa()
{
	int c = aa_getkey(ctx, 0);
	return (c == AA_ESC || c == 'q' || c == 'Q');
}

void Browser::refresh_msg_aa()
{
	aa_puts(ctx, 0, 0, AA_SPECIAL, msg_buf);
	aa_flush(ctx);
}

#endif /* aalib */
