// Copyleft (cl) 1999, 2000 by Malekith / The Final Crash of Corp. System
// <malekith@topnet.pl> / <http://malekith.topnet.pl>
// This is part of gallery image viewer. See file copying-left for details.
// Created on Jul 30 1999
// $Id: main.cc,v 2.16 1999/12/11 21:54:13 malekith Exp malekith $

#include "main.h"
#include "image.h"
#include "browser.h"

#include <unistd.h>

void clr(int x1, int y1, int x2, int y2);

/*
#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
*/


// globals

const int ups=1000;
int bytespp;
int bitspp;
extern "C" int getclock();
int screen_w, screen_h;
int white=0xff;

// Browser functions

Browser::Browser() : 
	quantize(DEF_QUANTIZE),
	dither(DEF_DITHER),
	fit_screen(true),
	quiet(false), verbose(false), debug(false), 
	screen_messages(true), show_message_screen(true), 
	fade_frames(7),
	scale(100), 
	scale_over_100(100), scale_below_100(10),
	black_and_white(false),
	aaflag(false),
#ifdef aalib
	ctx(0),
#endif
	
	last_action(-2*ups),
	orig_file_name(null),
	file_names(null),
	
	video_mode(-1), scroll_step(-1),
	gfx_is_dead(true),
	
	name(null),
	file_is_temp(false),
	msg_buf(0),
	next_pic(0)
{
	memset(screen_palette, 0, sizeof(screen_palette));
}

Browser::~Browser()
{
	kill_gfx();
}

void Browser::handle_dirty_rect()
{
	Rect image(on_screen.width(), on_screen.height());
	Rect under_text(0, 0, screen_w, 9);
	bool small = image.is_in(Rect(screen_w, screen_h));
	// small means centred
	if (small)
		image.move((screen_w - image.w()) / 2,
			   (screen_h - image.h()) / 2);


	if (!dirty.is_in(image)) {
		clr(dirty.x1, dirty.y1,  dirty.x2, dirty.y2);
		/*
		#define abs(a) ((a) > 0 ? (a) : (-a))
		if (dirty.x1 < image.x1)
			gl_fillbox(dirty.x1, dirty.y1 <? image.y1,
				   image.x1 - dirty.x1, abs(dirty.y2 - image.y2), 0);
		if (dirty.x2 > image.x2)
			gl_fillbox(image.x2, dirty.y1 <? image.y1,
				   dirty.x2 - image.x2, abs(dirty.y2 - image.y2), 0);
		if (dirty.y1 < image.y1)
			gl_fillbox(dirty.x1 <? image.x1, dirty.y1,
				   abs(dirty.x2 - image.x2), image.y1 - dirty.y1, 0);
		if (dirty.y2 > image.y2)
			gl_fillbox(dirty.x1 <? image.x1, image.y2,
				   abs(dirty.x2 - image.x2), dirty.y2 - image.y2, 0);
		*/
	}
	
	dirty = image;
	
	if (!under_text.is_in(image))
		clr(under_text.x1, under_text.y1, under_text.x2, under_text.y2);

	if (small) { on_screen.x = -image.x1; on_screen.y = -image.y1; }
	else {
		on_screen.x <?= on_screen.width()  - screen_w - 1;
		on_screen.y <?= on_screen.height() - screen_h - 1;
		on_screen.x >?= 0;
		on_screen.y >?= 0;
	}
	on_screen.put();
}

void Browser::update_screen(bool update_pal)
{
#	ifdef aalib
	if (aaflag) {
		Rect image(on_screen.width(), on_screen.height());	
		bool small = image.is_in(Rect(screen_w, screen_h));
		// small means centred
		if (small) {
			on_screen.x = on_screen.y = 0;
			on_screen.aaput((screen_w - on_screen.width()) / 2,(screen_h - on_screen.height()) / 2);
		} else
			on_screen.aaput(0,0);
		return;
	}
#	endif /* aalib */

	if (bitspp == 8 && update_pal)
		fade(black_palette);

	handle_dirty_rect();

	if (update_pal)
		set_palette();
}

void Browser::read()
{
	Image *loader = selector(orig_file_name);	
	loader->file_is_temp = file_is_temp;
	loader->file_name = name;
	loader->read();
	loader->clear_temp();
	buffer_stuff();
	draw_progress(0);
}

void Browser::show_and_recover()
{
	try {
		msgvf(_("loading %s...\n"), orig_file_name);
		on_screen.x = 0;
		on_screen.y = 0;

		read();

		scale = on_screen.width() * 100 / orig.width();
		update_screen(true);
		
		if (screen_messages)
			refresh_msg();
		
		msgvf(_("ok\n"));
	}
	catch (xRecover & e) {
		message(_("Error ``%s'' loading file ``%s''. "
			"Press space for next image.       "), e.msg, orig_file_name);
	}
}

void Browser::read_keys(int &key, int &shift)
{
#	ifdef aalib
	if (aaflag)
		read_keys_aa(key, shift);
	else
#	endif			/* aalib */
		read_keys_svga(key, shift);
}

void Browser::event_loop()
{
	int last = -1;
	int i = 0;
	bool dead = false;
	while (!dead) {
		int key, shift = 0;

		if (i >= nfiles)
			break;
		if (i != last) {
			last_action = getclock();
			orig_file_name = file_names[i];
			show_and_recover();
			last = i;
		}
		
		if (!aaflag)
			mouse_svga();
		else
			usleep(20000);
		
		if (next_pic) {
			i++;
			next_pic = 0;
			continue;
		}
		
		read_keys(key, shift);
		if (!key)
			continue;

		handle_arrows(key);

		if (getclock() - last_action > ups / 2) {
			switch (key) {
			case 0:
				break;
			case '\b':
				if (i > 0)
					i--;
				break;
			case ' ':
				i++;
				break;
			case '\n':
				toggle_fit_screen();
				break;
			case '+':
				scale_to(scale + (scale < 100 ?
						  scale_below_100 :
						  scale_over_100));
				break;
			case '-':
				scale_to(scale - (scale <= 100 ?
						  scale_below_100 :
						  scale_over_100));
				break;
			case '/':
				scale_to(scale-1);
				break;
			case '*':
				scale_to(scale+1);
				break;		
			case 'q':
				dead = true;
				break;
			case 'h':
			case '?':
				display_help();
				break;
			case '0' ... '9':
				{
					int n = key - '0';
					int s;
					if (shift)
						s = 10 * n;
					else
						s = 100 * n;
					scale_to(s);
				}
				break;
			case '\\':
				refresh_msg();
				break;
#			ifdef aalib
			default:
				if (aaflag)
					handle_aa_key(key);
				break;
#			endif
			}
		}
	}
}

void Browser::run(int argc, char ** argv)
{
#	ifdef HAVE_LOCALE_H
	setlocale(LC_ALL, "");
#	endif
#	ifdef HAVE_LIBINTL_H
	bindtextdomain(NAME, LOCALEDIR);
	textdomain(NAME);
#	endif
#	ifdef HAVE_LOCALE_H
	setlocale(LC_ALL, "");
#	endif
	handle_config(argc,argv);
	init_gfx();
	event_loop();
	kill_gfx();
	char ** p = file_names;
	while(*p) delete [] *p++;
	delete [] file_names;
}

bool Browser::progress_bar(int a, int b)
{
	if (a < 0 && b < 0) {
		a = -a;
		b = -b;
	}
	
	int np = (a >? 0) * 100 / (b >? 1);
	np <?= 100;
	np = (subrange_r - subrange_l) * np / 100 + subrange_l;
	
	int t = getclock();
	if (t - last_updated > 100 || t - last_updated < 0) {
		last_updated = t;
		draw_progress(act_progress = np);
	}

#	ifdef aalib
	if (aaflag)
		return get_halt_code_aa();
	else 
#	endif /* aalib */
		return get_halt_code_svga();
}

void Browser::draw_progress(int perc)
{
#	ifdef aalib
	if (aaflag)
		draw_progress_aa(perc);
	else
#	endif
		draw_progress_svga(perc);
}

void Browser::refresh_msg()
{
	free(msg_buf);
	asprintf(&msg_buf, "%s (%d%%)", orig_file_name, scale);
#	ifdef aalib
	if (aaflag)
		refresh_msg_aa();
	else
#	endif
		refresh_msg_svga();
	
}

void Browser::wait_key()
{
	int key,shift;
	usleep(500000);
	
	do {
		read_keys(key, shift);
		usleep(100000);
	} while (key == 0);
}

// ---------------------------------------------------------------------------
// main()
// ---------------------------------------------------------------------------
extern bool _aaflag;
Browser * browser_ptr;
void pre_init_gfx(int &argc, char ** &argv);

int main(int argc, char **argv)
{
	pre_init_gfx(argc, argv);
	try {
		Browser browser;
		browser_ptr = &browser;
		browser.aaflag = _aaflag;
		browser.run(argc, argv);
	} catch (xcpt & e) {
		fprintf(stderr, "%s: '%s'\n", e.name(), e.msg);
		exit(1);
	}
	catch (...) {
		fprintf(stderr, "unknown unhandled xcpt\n");
		abort();
	}
	return 0;
}

